open Automaton_state
open Automaton_arrow
open Automaton_bundle
open Utils
 
module type AutomatonSig =
sig
  type statetype
  and arrowtype
  and bundletype
  and t
  val states : t -> statetype list
  val arrows : t -> arrowtype list
  val start : t -> statetype
  val bundle : t -> statetype -> bundletype
  val create : statetype -> statetype list -> arrowtype list -> t
  val print : t -> unit
end


module Automaton =
  functor(State : StateSig) ->
  functor(Arrow : ArrowSig with type statetype = State.t) ->
  functor(Bundle: BundleSig with type arrowtype = Arrow.t) ->
struct
  type statetype = State.t
  type arrowtype = Arrow.t
  type bundletype = Bundle.t
  type t = Automaton of statetype * statetype list * arrowtype list
  let  states = function Automaton(start, states, arrows) -> states
  let  arrows = function Automaton(start, states, arrows) -> arrows
  let  start  = function Automaton(start, states, arrows) -> start
  let  bundle = function Automaton(start, states, arrows) ->
    fun (q:statetype) ->
      (Bundle.from_arrow_list (List.filter (fun a -> State.compare q (Arrow.source a) == 0) arrows))
  let  create start states arrows = Automaton(start, states, arrows)
  let  print (a : t) =
    State.print_start (start a);
    print_newline();
    List.iter State.print (states a);
    print_newline();
    List.iter Arrow.print (arrows a)
end


