open Automaton_state

module type ArrowSig =
sig
  type labeltype
  and statetype
  and t
  val source : t -> statetype
  val target : t -> statetype
  val label : t -> labeltype
  val compare : 'a -> 'a -> int
  val compose : t -> t -> t
  val create : statetype -> labeltype -> statetype -> t
  val print : t -> unit
end

module Arrow =
  functor(State : StateSig) ->
struct
  type labeltype = string
  type statetype = State.t
  type t = Arrow of statetype * labeltype * statetype
  let create s l t = Arrow(s,l,t)
  let source = function Arrow(s,l,t) -> s
  let target = function Arrow(s,l,t) -> t
  let label  = function Arrow(s,l,t) -> l
  let compare= compare
  let compose ar1 ar2 =
    match ar1,ar2 with
      | Arrow(s1,l1,t1), Arrow(s2,l2,t2) ->
	  if (State.compare t1 s2) == 0
	  then Arrow(s1,l1^l2,t2)
	  else failwith "Error: not composable arrows"
  let print = function Arrow(s,l,t) ->
    Printf.fprintf stdout "%s -> %s\t%s\n" 
      (State.id s) (State.id t) l
end
