
open Automaton_state
open Automaton_arrow
open Automaton_bundle
open Automaton_bisimulation
open Automaton_
open Utils

module type BlockSig =
sig
  type statetype
  and bundletype
  and automatontype
  and buckettype
  and resulttype
  and t

  val from_states : statetype list -> t
  val id : t -> string
  val states : t -> statetype list
  val norm : t -> bundletype
  val compare : t -> t -> int
  val split  : bundletype -> (statetype -> resulttype option) -> t -> buckettype * t
  val cardinal : t -> int
  val close_block : automatontype -> (statetype -> t) -> buckettype -> t
  val to_state : int -> t -> State.t
  val next : (statetype -> t) -> (t -> statetype) -> bundletype -> bundletype
  val mem : statetype -> t -> bool
end


module Block =
  functor(State : StateSig) ->
  functor(Arrow : ArrowSig with type statetype = State.t) ->
  functor(Bundle: BundleSig with type statetype = Arrow.statetype and type arrowtype = Arrow.t) ->
  functor(Automaton : AutomatonSig with type statetype = State.t and type arrowtype = Arrow.t and type bundletype = Bundle.t) ->
  functor(Bisimulation : BisimulationSig with type bundletype = Bundle.t) ->

struct

  type statetype = State.t
  type bundletype = Bundle.t
  type automatontype = Automaton.t
  type t = Block of string * statetype list * bundletype
  type buckettype = t
  type resulttype = Bisimulation.resulttype

  let from_states states = Block("", states, (Bundle.from_arrow_list []))
  let create id states norm = Block(id, states, norm)
  let id = function Block(name, states, norm) -> name
  let states = function Block(name, states, norm) -> states
  let to_state n block =
    State.create ("b" ^ (string_of_int n))
  let norm   = function Block(name, states, norm) -> norm
  let compare block1 block2 = Bundle.compare (norm block1) (norm block2)
  let next env h_n bundle =
    Bundle.from_arrow_list
      (unique
	 (List.map
	    (fun ar -> Arrow.create (Arrow.source ar) (Arrow.label ar) (h_n (env (Arrow.target ar))))
	    (bundle)))

  let split minimal pred block =
    let (blk1, blk2) = List.partition (fun x -> (pred x) <> None) (states block) in
      ( (create "" blk1 minimal), (create (id block) blk2 (norm block)))

  let cardinal block = List.length (states block)

  let close_block env bucket = bucket
  let mem state block = List.mem state (states block)
end
