
open Automaton_state
open Automaton_arrow
open Utils

module type BundleSig =
sig
  type arrowtype
  and statetype
  and quadtype
  and t = quadtype list
  val from_arrow_list : arrowtype list -> t
  val to_arrow_list : statetype -> t -> arrowtype list
  val normalize : (t -> t) -> t -> t
  val minimize : (t -> t) -> t -> t
  val diff : t -> t -> t
  val compare : t -> t -> int
  val print : t -> unit
end


module Bundle =
  functor (State: StateSig) ->
  functor (Arrow: ArrowSig with type statetype = State.t) ->
struct
  type statetype = State.t
  type arrowtype = Arrow.t
  type quadtype = arrowtype
  type t = quadtype list


  let from_arrow_list = 
    function x -> x

  let rec to_arrow_list q = 
    function
	[] -> []
      | a::arws -> (Arrow.create q (Arrow.label a) (Arrow.target a )) :: (to_arrow_list q arws)

  let normalize = fun red x -> x
  let minimize = fun red x -> x
  let diff = list_diff
  let compare bl1 bl2 =
    let xx = State.create "dummy" in
    let bl1' = List.sort Arrow.compare (List.map (fun ar -> Arrow.create xx (Arrow.label ar) (Arrow.target ar)) (bl1)) in
    let bl2' = List.sort Arrow.compare (List.map (fun ar -> Arrow.create xx (Arrow.label ar) (Arrow.target ar)) (bl2)) in
      compare bl1' bl2'
  let print bundle = List.iter Arrow.print (bundle)
end

