

/* $Id$ */

%{

open Ast
open Parserdecls

%}

%token <int> INT
%token <string> ID 
%token EOF

%token DEFINE
%token BUILD
%token WRITE_HD
%token CONST
%token NIL
%token TAU
%token PIPE
%token DOT
%token PLUS
%token COMMA
%token EQ
%token IN
%token OUT
%token LPAREN
%token RPAREN
%token LBRACKET
%token RBRACKET


%left PIPE
%left PLUS
%left DOT

%start commands

%type< Ast.command list > commands
%type< Ast.command > command
%type< Ast.pi_term > pi_term

%%

commands
        : 
	{ [] }
        | commands command
	{ $1 @ [$2] }
        ;

command 
        : DEFINE ID LPAREN param RPAREN EQ pi_term
	{ Ast.Define( $2, $4, $7 ) }
        | BUILD ID 
	{ Ast.Build( $2 ) }
        | WRITE_HD ID 
	{ Ast.BuildHD( $2 ) }
        | CONST ID
	{ Ast.Const( $2 ) }
	;

//===========================================================================

pi_term 
	: NIL
	{ Ast.Nil }
        | ID IN LPAREN ID RPAREN DOT pi_term
	{ Ast.BoundInput( $1, $4, $7 ) }
        | ID OUT ID DOT pi_term
	{ Ast.FreeOutput( $1, $3, $5 ) }
        | TAU DOT pi_term
	{ Ast.Tau( $3 ) }
        | pi_term PIPE pi_term
	{ Ast.Par( $1, $3 ) }
        | PIPE LPAREN pi_term_list RPAREN
	{ Ast.ParList( $3 ) }
        | pi_term PLUS pi_term
	{ Ast.Sum( $1, $3 ) }
        | PLUS LPAREN pi_term_list RPAREN
	{ Ast.SumList( $3 ) }
        | LPAREN ID RPAREN pi_term %prec DOT
	{ Ast.Restr( $2, $4 ) }
        | LBRACKET ID EQ ID RBRACKET pi_term %prec DOT
	{ Ast.Match( $2, $4, $6 ) }
        | ID LPAREN param RPAREN
	{ Ast.Proc( $1, $3 ) }
        | LPAREN pi_term RPAREN
	{ $2 }
        ;

param   :
	{ None }
        | param_aus
	{ Some $1 }
        ;

param_aus 
        : ID                 
	{ [$1] }
        | param_aus COMMA ID
	{ $1 @ [$3] }
        ;

pi_term_list 
        : pi_term                  
	{ [$1] }
        | pi_term_list COMMA pi_term
	{ $1 @ [$3] }
        ;

%%


