
open Ast
open Printf
open Parserdecls

(* [apply roo id] returns the state in the environment roo that
 * is associated with the name id
 *)
let apply roo id =
  try
    (List.assoc id roo)
  with Not_found -> failwith ("apply: not found " ^ id)

(* [read_file filename] parse the contents of file filename and returns 
 * the abstract syntax tree
 *)
let read_file filename =
  let lexbuf = Lexing.from_channel (open_in filename)
  in Parser.main Lexer.token lexbuf

let read_stdin () =
  let lexbuf = Lexing.from_channel stdin
  in Parser.main Lexer.token lexbuf


let print_label f (act, pi, sigma) =
  match act with
    | "tau" ->
	begin
          fprintf f "[ label = \"tau\t";
	  Utils.print_int_list f sigma;
          fprintf f "\" ]\n"
	end
    | "in" ->
	begin
          fprintf f "[ label = \"in\t";
          Utils.print_int_list f pi;
          fprintf f " - ";
          Utils.print_int_list f sigma;
          fprintf f "\" ]\n"
	end
    | "out" ->
	begin
          fprintf f "[ label = \"out\t";
          Utils.print_int_list f pi;
          fprintf f " - ";
          Utils.print_int_list f sigma;
          fprintf f "\" ]\n"
	end
    | "bin" ->
	begin
          fprintf f "[ label = \"bin\t";
          Utils.print_int_list f pi;
          fprintf f " - ";
          Utils.print_int_list f sigma;
          fprintf f "\" ]\n"
	end
    | "bout" ->
	begin
          fprintf f "[ label = \"bout\t";
          Utils.print_int_list f pi;
          fprintf f " - ";
          Utils.print_int_list f sigma;
          fprintf f "\" ]\n"
	end
    | _ -> failwith ("expected a pi-label found " ^ act)


let print_arrow f ast =
  match ast with
    | DeclArrow( source, target, pi_label ) ->
	fprintf f "\t%s\t->\t%s\t[fontname=hevetica]\t" source target;
	print_label f pi_label;
    | _ -> failwith "expected an arrow"

let print_arrow_list f ast =
  match ast with
    | DeclArrowList( decl_arrows ) ->
	List.iter (print_arrow f) decl_arrows
    | _ -> failwith "expected arrow list"

let print_hd f ast = 
  match ast with
    | DeclAutoma( decl_start, decl_states, decl_arrows ) ->
	begin
	  fprintf f "digraph HD {\n";
	  print_arrow_list f decl_arrows;
	  fprintf f "}\n"
	end
    | _ -> failwith "expected an automaton"


let _ =
  
  try
    let ast = read_stdin () in
      print_hd stdout ast
  with
      Failure s -> 
	(fprintf stderr "failure: %s\n" s); (exit ~-1)
    | Invalid_argument s -> 
	(fprintf stderr "usage: hd-to-dot filename\n\n"); (exit ~-1)
    | Lexer.Eof ->
	(fprintf stderr "error: unexpected end of file\n"); (exit ~-1)
    | Parsing.Parse_error ->
	(fprintf stderr "error: syntax error at line %d\n" !line_number); (exit ~-1)



