/* parser.mly - (C) 2002 by Roberto Raggi, Emilio Tuosto */

/* $Id: parser.mly,v 1.1 2002/07/19 09:15:53 raggi Exp $ */

%{

open Ast
open Parserdecls
open Utils

%}

%token <int> INT
%token <string> ID 
%token EOL 
%token ARROW
%token COMMA
%token SEMICOLON
%token LPAREN
%token RPAREN
%token LBRACKET
%token RBRACKET
%token LBRACE
%token RBRACE
%token STATE
%token START
%token STAR
%token EQ
%token DOT
%token EOF
%token TAU
%token BIN
%token BOUT
%token IN
%token OUT
%token IDENTITY
%token EXCHANGE


%start main

%type <Ast.decl> main

%%

main	
	: decl_start decl_state_list decl_arrow_list EOF 
          {
	    DeclAutoma($1, $2, $3) 
	  }
	;

decl_start
	: START ID			{ DeclStart $2 }
	;

group_opt
        :                                                   { None }
        | LBRACKET group semicolon_opt RBRACKET              { Some $2 }
        ;

semicolon_opt
        :             { None }
        | SEMICOLON   { None }
        ;

group   : LBRACKET name_list RBRACKET          { [$2] }
        | group SEMICOLON LBRACKET name_list RBRACKET            { $1 @ [ $4 ] }
        ;

decl_state_list
	: decl_state 			{ DeclStateList [$1] }
	| decl_state_list decl_state	
	    { 
	      match $1 with 
		  DeclStateList(l) -> DeclStateList(l @ [$2])
		| _ -> failwith "!!!" 
	    }
	;

decl_state
        : STATE ID INT group_opt
          { 
	    match $4 with
	      | None -> DeclState($2, (range 1 ($3+1)), [(range 1 ($3+1))])
	      | Some gr -> DeclState($2, (range 1 ($3+1)), gr)
	  }
	;

name	
	: INT				{ $1 }
	| STAR				{ 0 }
	;
	
name_list_opt
        :                               { [] }
        | name_list                     { $1 } 
        ;

name_list
  : name_list2 semicolon_opt { $1 }
  ;

name_list2
	: name		{ [$1] }
	| name_list2 SEMICOLON name	{ $1 @ [$3] }
	;

sigma
  : name_list_opt { $1 }
  ;

pi
      : LBRACKET name_list RBRACKET { $2 }
      ;

pi_label
        : TAU LBRACKET sigma RBRACKET	                { "tau", [], $3 }
        | BIN pi LBRACKET sigma RBRACKET                { "bin", $2, $4 }
        | BOUT pi LBRACKET sigma RBRACKET               { "bout", $2, $4 }
        | IN pi LBRACKET sigma RBRACKET	                { "in",  $2, $4 }
        | OUT pi LBRACKET sigma RBRACKET                { "out", $2, $4 }
	;

decl_arrow
	: ID ARROW ID pi_label { DeclArrow($1, $3, $4) }
	;

decl_arrow_list
 	: decl_arrow 					{ DeclArrowList [$1] }
	| decl_arrow_list decl_arrow 			
	    { 
	      match $1 with
		  DeclArrowList l -> DeclArrowList(l @ [$2])
		| _ -> failwith "expect decl arrow list!!"
	    }
	;


%%


