
open Utils;;
open Ast;;
open Hdxml;;
open Parserdecls

open Pxp_types;;
open Pxp_document;;
open Pxp_yacc;;
open World;;

(* [apply roo id] returns the state in the environment roo that
 * is associated with the name id
 *)
let apply roo id =
  try
    (List.assoc id roo)
  with Not_found -> failwith ("apply: not found " ^ id)

let convertName id =
  if String.contains id '"' 
  then String.sub id 1 ((String.length id) - 2)
  else id

(* [create_states dec] returns list of pair (id, state), where
 * the list is created from the declaration dec
 *)
let create_states dec =
  match dec with
      DeclStateList(l) ->
	(List.map 
	   (function 
		DeclState (id, names, group) ->  
		  ((convertName id), HDState.create (convertName id) names group)
	      | _ -> failwith "create_states: internal error!!")
	   l)
    | _ -> failwith "create_states: internal error!!"


(* [create_arrows dec roo] returns a list of arrows created from
 * the declaration dec and environment roo
 *)
let create_arrows dec roo = 
  match dec with
    | DeclArrowList(l) -> 
	(List.map 
	   (function 
		DeclArrow(id1,id2,label) ->
		  HDArrow.create (apply roo (convertName id1)) label (apply roo (convertName id2))
	      | _ -> failwith "create_arrows: internal error!!")
	   l)
    | _ -> failwith ""

		      
(* [create_start dec roo] returns a state created from the declaration
 * dec and the environment roo
 *)
let create_start dec roo = 
  match dec with
    | DeclStart(id) -> (apply roo (convertName id))
    | _ -> failwith "create_start: internal error!!"


(* [create_automata dec] returns an hdautoma from declaration dec *)
let create_automata = 
  function 
    | DeclAutoma(decl_start, decl_states, decl_arrows) -> 
	let roo = (create_states decl_states) in	  
	let arrows = (create_arrows decl_arrows roo) in
	let start = (create_start decl_start roo) in
	  HD.create 
	    start
	    (snd (List.split roo))
	    arrows
    | _ -> failwith "create_automa: internal error!!!"


(* [read_file filename] parse the contents of file filename and returns 
 * the abstract syntax tree
 *)
let read_file filename =
  let lexbuf = Lexing.from_channel (open_in filename)
  in Parser.main Lexer.token lexbuf

let read_stdin () =
  let lexbuf = Lexing.from_channel stdin
  in Parser.main Lexer.token lexbuf


let main() =
  let file = (from_channel stdin) in    
      (* Printf.fprintf stdout "Mihda XML - Version 0.2\n(C) 2002,2003 by Dipartimento di Informatica di Pisa\n\n"; *)
      let hda = create_automata (read_stdin ()) in
        print_hdxml stdout hda
;;


main();;

