
open Hd_state
open Utils
open Printf

module type ArrowSig =
sig
  type labeltype = string * int list * int list
  and statetype
  and t
  val create : statetype -> labeltype -> statetype -> t
  val source : t -> statetype
  val target : t -> statetype
  val label : t -> labeltype
  val sigma : t -> int list
  val pi : t -> int list
  val names : t -> int list
  val pilabel : t -> string
  val print_label : out_channel -> labeltype -> unit
  val print_arrow : out_channel -> t -> unit
  val print : t -> unit
  val compare : t -> t -> int
  val compose : t -> t -> t
end


module Arrow =
  functor(State : StateSig) ->
struct
  type labeltype = string * int list * int list
  and statetype = State.t
  and t =  Arrow of statetype * labeltype * statetype

  let create s l t = Arrow(s, l, t)

  (* source ar returns the source of the arrow ar *)
  let source = function Arrow( source, _, _ ) -> source

  (* target ar returns the destination of the arrow ar *)
  let target = function Arrow( _, _ , target ) -> target

  (* label ar returns the label of the arrow ar *)
  let label = function Arrow( _, label, _ ) -> label

  (* sigma ar returns the sigma of the label of ar *)
  let sigma ar = match (label ar) with (_,_,sigma) -> sigma

  let pi ar = match (label ar) with (_,pi,_) -> pi

  let names ar = Utils.unique ((pi ar) @ (sigma ar))

  let pilabel ar  = match (label ar) with (l,_,_) -> l

  (* printint a label *)
  let print_label f =
    function (pilabel, pi, sigma) ->
      fprintf f "\t%s" pilabel;
      if pilabel <> "tau" then begin
	(print_int_list f pi);
      end;
      (fprintf f "\t");
      (print_int_list f sigma)

  (* printing an arrow *)
  let print_arrow f =
    function 
	Arrow( s, l, d ) ->
	  (fprintf f "\"%s\"\t->\t\"%s\"\t" (State.id s) (State.id d));
	  (print_label f l);
	  (fprintf f "\n" )
	  
  let print = print_arrow stdout
		
  let compare = compare
		  
  let compose ar1 ar2 = ar1 
end

