
open Hd_state
open Hd_arrow
open Hd_bundle
open Utils
open Printf


module type AutomatonSig =
sig
  type statetype
  and arrowtype
  and bundletype
  and t
  val states : t -> statetype list
  val arrows : t -> arrowtype list
  val start : t -> statetype
  val bundle : t -> statetype -> bundletype
  val create : statetype -> statetype list -> arrowtype list -> t
  val print : t -> unit
end

(**********************************************************************************************)


module Automaton =
  functor (State : StateSig) ->
  functor (Arrow : ArrowSig with type statetype = State.t) ->
  functor (Bundle: BundleSig with type statetype = Arrow.statetype and type arrowtype = Arrow.t and type labeltype = Arrow.labeltype) ->
struct
    type statetype = State.t
    and arrowtype = Arrow.t
    and bundletype = Bundle.quadtype list
    and t = (statetype * statetype list * arrowtype list)

    let states (s,states,arrows) = states

    let arrows  (s,states,arrows) = arrows

    let start  (s,states,arrows) = s

    let bundle hda s =
      Bundle.from_arrow_list (List.filter (fun x -> 0 = State.compare (Arrow.source x) s) (arrows hda))

    let create s states arrows = (s, states, arrows)
    let print hda =
      State.print_start stdout (start hda);
      print_newline ();
      List.iter State.print (states hda);
      print_newline ();
      List.iter Arrow.print (arrows hda);
end


