
open Hd_bundle
open Utils

module type BisimulationSig =
sig
  type bundletype
  type resulttype

  val bisimilar: bundletype -> bundletype -> resulttype option
end

module Bisimulation =
  functor(Bundle : BundleSig) ->
struct
  type bundletype = Bundle.quadtype list
  and resulttype = (int * int) list

  let select bundle lab = List.filter (fun q -> (Bundle.label q) = lab) bundle 

  let bisimilar_real minimal norm =
    let check pair_list =											
      let (dom, cod) = List.split pair_list in
	(List.length dom == List.length (unique dom)) in
    let check_bij pair_list =										
      let l = unique pair_list in
	(check l) && (check (invert l)) in
    let b1 =
      List.map
	(select (List.sort (fun a b -> compare (Bundle.label a) (Bundle.label b)) minimal))
	[ "tau"; "out"; "bout"; "in"; "bin" ] in
    let b2 =
      List.map
	(select (List.sort (fun a b -> compare (Bundle.label a) (Bundle.label b)) norm))
	[ "tau"; "out"; "bout"; "in"; "bin" ] in
    let n1 = List.length ( Bundle.active_names minimal )  in
    let n2 = List.length ( Bundle.active_names norm ) in
    let cond = ((compare (List.map List.length b1) (List.map List.length b2)) = 0) && (n1 = n2) in
      if cond
      then

	let compute_rho qd1 qd2 =
	  if (Bundle.target qd1) <> (Bundle.target qd2)
	  then None
	  else
	    let pairs =  List.combine (Bundle.names qd1) (Bundle.names qd2) in				
	      if check_bij pairs									
	      then Some pairs
	      else None in

	let rec find_rho_quad qd = function
	  | [] -> ( None, [] )
	  | qd'::qds ->
	      match compute_rho qd qd' with
		| Some(r) -> ((Some r), qds)
		| None ->  
		    let r = (find_rho_quad qd qds) in
		      ((fst r), qd' :: (snd r)) in

	let rec theta_ bl1 bl2 =									
	  match bl1 with
	    | [] -> []
	    | qd::qds -> 
		let ( r, bl2rest ) = ( find_rho_quad qd bl2 ) in				
		  match r with 
		    | Some rho -> rho @ ( theta_ qds bl2rest )
		    | None -> failwith "Block.theta.theta_: minimal differs from norm" in

	let constraints = 
	  (try 
	     Some (unique (List.flatten ( List.map2 theta_ b1 b2 ) ))
	   with Failure e -> None) in
	  
	  match constraints with
	      None -> None
	    | Some res ->
		if check_bij res
		then constraints
		else None

      else None

  let bisimilar minimal norm =
    try
      bisimilar_real minimal norm
    with _ -> failwith "fine!"

end
