open Printf
open Utils

module type StateSig =
sig
  type t
  val id : t -> string
  val names : t -> int list
  val create : string -> int list -> int list list -> t
  val star : t
  val group : t -> int list list
  val cardinal : t -> int
  val print_start : out_channel -> t -> unit
  val print_state : out_channel -> t -> unit
  val print : t -> unit
  val compare : 'a -> 'a -> int
end



module State =
struct
    type t =
      | Star
      | State of string * int list * int list list

    let id =
      function 
	| Star              -> "*"
	| State( id, _, _ ) -> id

    let names = 
      function 
	| Star                 -> []
	| State( _, names, _ ) -> names

    let create id names group = State(id, names, group)

    let star = Star

    (* group s return the permutation group of the state s *)
    let group = 
      function 
	| Star                 -> []
	| State( _, _, group ) -> group


    (* cardinal s returns how many names have the state s *)
    let cardinal =
      function
	| Star                 -> 0
	| State( _, names, _ ) -> List.length names


    (* printing the name of a state *)
    let print_start f = 
      function
	| State( id, _, _ ) -> (fprintf f "start \"%s\"\n" id)
	| Star              -> (fprintf f "start *\n")


    (* printing a state *)
    let print_state f =
      function 
	| State( id, names, group ) ->
	    (fprintf f "state\t\"%s\"\t\t%d" id (List.length names)); 
	    (fprintf f "\t");
	    (match group with 
		  | [ [] ]  -> fprintf f "\n"
		  | _ ->
	    (fprintf f "%s" "[ ");
	    (List.iter 
	       (fun x-> (print_int_list f x); fprintf f "%s" " ; ")
	       group);
	    (fprintf f "%s" " ]\n"))
	| Star ->
	    (fprintf f "state *\n")

    let print = print_state stdout

    let compare = compare
  end
