(* lexer.mll - (C) 2002 by Roberto Raggi, Emilio Tuosto *)

(* $Id: lexer.mll,v 1.1.1.1 2002/06/07 15:44:07 etuosto Exp $ *)

{
open Parser
open Parserdecls
exception Eof  
}

rule token = parse
    [ ' ' '\t' ]		 { token lexbuf }
  | '\n'                         { line_number := !line_number + 1;
                                   token lexbuf }
  | '#' [ ^ '\n' ]*		 { token lexbuf }
  | "id"                         { IDENTITY }
  | "exch"                       { EXCHANGE }
  | "state"			 { STATE }
  | "start"			 { START }
  | "tau"			 { TAU }
  | "bin"			 { BIN }
  | "bout"			 { BOUT }
  | "in"			 { IN }
  | "out"			 { OUT }
  | [ '0'-'9' ]+		 { INT(int_of_string(Lexing.lexeme lexbuf)) }
  | [ 'A'-'Z' 'a'-'z' '0'-'9' ]+ { ID(Lexing.lexeme lexbuf) }
  | '"' [ ^ '"' ]+ '"'           { ID(Lexing.lexeme lexbuf) }
  | "->"			 { ARROW }
  | '*'				 { STAR }
  | ','				 { COMMA }
  | '.'                          { DOT }
  | ';'				 { SEMICOLON }
  | '('				 { LPAREN }
  | ')'				 { RPAREN }
  | '['				 { LBRACKET }
  | ']'				 { RBRACKET }
  | '{'				 { LBRACE }
  | '}'				 { RBRACE }
  | '='				 { EQ }
  | eof				 { EOF }
