(*s The interface for arrow is given below.
  * Only the minimal features of arrows are
  * included.
  * This module specifies a type [t] which is
  * a dependent type. Indeed, it depends on
  * the type of the states and the type of
  * the labels.
*)

module type Arrow =
sig

  (*s is the type of states.
    *
    * @see State
   *)
  type statetype

  (*s is the type of labels.
    *
    * @note In this prototype the type of label is a generic type. Indeed, no interface is defined for labels. Probably in future it would be refined.
   *)
  type labeltype

  (*s is the type of the arrows.
    * It is not specified and depends on
    * [statetype] and [labeltype].
   *)
  type t

  (*s returns the source of the arrow [arw].
    *
    * @arg arw is the arrow
   *)
  val source : arw:t -> statetype

  (*s returns the target (destination) of the arrow [arw].
    *
    * @arg arw is the arrow
   *)
  val target : arw:t -> statetype

  (*s returns the label of the arrow [arw].
    *
    * @arg arw is the arrow
   *)
  val label : arw:t -> labeltype

  (*s compares arrows [arw1] and [arw2].
    * The result is
    *  @il
    *    @i 0, if the arrows are equal,
    *    @i -1, if [arw1] is less than [arw2],
    *    @i 1, otherwise.
    *  @li
    *
    * @arg [arw1] is the first arrow
    * @arg [arw2] is the second arrow
    *
    * @note The comparison is meant to be a structural comparison. However, any function that does not equate two conceptually different states may be adopted.
  *)
  val compare : arw1:t -> arw2:t -> int

  (*s returns an arrow from [source arw1] to [target arw2],
    * [arw1] and [arw2] are arrows.
    *
    * @arg [arw1] is the first arrow
    * @arg [arw2] is the second arrow
    *
    * @note Up to now [compose] is not used; it has been included in the interface, because in future extensions of HD-automata it could be useful, e.g. for specifying weak version of semantics.
   *)
  val compose : arw1:t -> arw2:t -> t

  (*s prints the arrow [arw] on the standard output.
    *
    * @arg arw is the arrow to be printed
   *)
  val print : arw:t -> unit

end
