(*s The following module defines the interface for automata type.
  * An automaton is built out from states and arrows between states.
  * However, also the type of a bundle should be provided for
  * specifying automata.
  * The functions of an automaton allows to extract the relevant
  * information.
 *)

module type Automaton =
sig
  (*s is the type of the states in the automaton.
    *)
  type statetype

  (*s is the type of the arrows in the automaton.
    *)
  type arrowtype

  (*s is the type of the bundle ([all abservable actions]).
    *)
  type bundletype

  (*s is the type for automata.
    *)
  type t

  (*s creates an automaton from [start], [states] and [arrows].
    *
    * @arg start is the starting state
    * @arg states are the states of the automaton
    * @arg arrows are the arrows of the automaton
    *)
  val  create : start:statetype ->  states:statetype list -> arrows:arrowtype list -> t

  (*s returns the start state of the automaton [a].
    *
    * @arg a is an automaton
   *)
  val  start : a:t -> statetype

  (*s returns the list of states of the automaton [a].
    *
    * @arg a is an automaton
    *)
  val  states : a:t -> statetype list

  (*s returns the list of arrows of the automaton [a].
    *
    * @arg a is an automaton
    *)
  val  arrows : a:t -> arrowtype list

  (*s returns the bundle of the state [state]
    * in automaton [a].
    *
    * @arg a is an automaton
    * @arg state is the departing state
    *)
  val  bundle : a:t -> state:statetype -> bundletype

  (*s prints the automaton on the standard output.
    *
    * @arg a is an automaton
    *)
  val  print  : a:t -> unit
end
