(*s The Bisimulation module specifies the interface for expressing
  * the behavioural equivalence the user is interested in.
  * Note that this module depends on the type adopted for bundles
  * and is also parameterized with respect to the type of the result.
  * The idea is that, in some cases it is not enough to know that
  * the relation holds for two bundle but also auxiliary informations
  * may be useful.
  * For instance, in the case of standard automata, [resulttype]
  * could be simply [bool], but automata for name passing calculi
  * also has names appearing on bundles and name correspondences
  * could be used for computing the minimal automaton.
  *
  * @see hd_bisimulation.ml
 *)
module type Bisimulation =
sig

  (*s is the type of bundles.
   *)
  type bundletype

  (*s is the type of the result of [bisimilar].
   *)
  type resulttype

  (*s returns an optional type; if the relation holds
    * between [bl1] and [bl2], then [res] should be [Some(r)]
    * (for some [r] of type [resultype]) otherwise [None] is
    * returned.
    *
    * @arg [bl1] is the first bundle
    * @arg [bl2] is the second bundle
   *)
  val bisimilar : bl1:bundletype -> bl2:bundletype -> resulttype option 

end
