(*s  The module State defines the interface of automata states.
 *)

module type State =
sig
  (*s is the type of the states.
    *)
  type t

  (*s returns the identifier of the state [state].
    *
    * @note It is assumed that state identifiers are uniquely defined.
    *
    * @arg state is the state
    *)
  val id : state:t -> string

  (*s compares the two states [st1] and [st2].
    * The result is
    * @il
    *    @i 0, if the arrows are equal,
    *    @i -1, if [st1] is less than [st2],
    *    @i 1, otherwise.
    * @li
    *
    * @arg st1 is the first state
    * @arg st2 is the second state
    *
    * @note The comparison is meant to be a structural comparison. However, any function that does not equate two conceptually different states may be adopted.
    *)
  val compare : st1:t -> st2:t -> int

  (*s prints [state] on the standard output.
    *
    * @arg state is the state
    *)
  val print : state:t -> unit

end
